import {AdminPages, TopBarButtons} from "@/js/admin/admin_pages";
import {Api} from "@/js/admin/Api";
import {Products, ProductsModel, Stock, StockModel} from "@/js/declarations";
import {ApiHost} from "@/js/vars";
import {MDCDialog} from "@material/dialog";
import {MDCRipple} from "@material/ripple";
import {MDCTextField} from "@material/textfield";
import {MDCSelect} from '@material/select';

export class PageProducts {
    private _appContainer: HTMLElement;
    private _context: AdminPages;
    private _api: Api;
    private _pageContainer?: HTMLElement;
    private _dialog?: MDCDialog;
    private _dialogStruct?: DialogStruct;
    private _pendingAction: boolean;
    private _dialogEls?: DialogEls;

    constructor(appContainer: HTMLElement, adminPages: AdminPages) {
        this._appContainer = appContainer;
        this._api = new Api();
        this._context = adminPages;
        this._pendingAction = false;
        this._context.setTitle("Produtos");
        this.buildPage();
    }

    public destroy(): void {
        this._dialog?.root.remove();
        document.getElementById("btn-add-produto")?.remove();
    }

    private async buildPage() {
        const htmlData = await fetch("/template_produtos");
        const htmlDataBody = await htmlData.text();
        this._appContainer.innerHTML = htmlDataBody;

        const dialogHtmlData = await fetch("template_produtos_dialog");
        const dialogHtmlDataBody = await dialogHtmlData.text();

        const dialogEl = document.createElement('div');
        dialogEl.className = "mdc-dialog mdc-dialog--fullscreen";
        dialogEl.style.zIndex = "1000";
        dialogEl.innerHTML = dialogHtmlDataBody;
        dialogEl.id = "dialog-produtos";

        const docBody = document.querySelector('body');

        if (docBody) {
            docBody.insertAdjacentElement('afterbegin', dialogEl);
        }

        this._dialog = new MDCDialog(document.getElementById('dialog-produtos') || new HTMLElement());

        this._dialog.listen('MDCDialog:closing', () => {
            if (!this._pendingAction) {
                this.destroy();
                this.buildPage();
            }
        });

        this._dialogStruct = {
            header: document.getElementById("txt-head-dialog") || new HTMLElement(),
            body: document.getElementById("produtos-dialog-body") || new HTMLElement(),
            buttonOk: document.getElementById("button-dialog-ok") || new HTMLElement(),
            buttonOkLabel: document.getElementById("btn-label-dialog") || new HTMLElement()
        };

        this._pageContainer = document.querySelector('.container-gerenciador-produtos') || new HTMLElement();

        const buttons: TopBarButtons[] = [
            {
                label: "Adicionar Produto",
                id: "btn-add-produto",
                icon: "add"
            }
        ]
        this._context.setButtons(buttons);
        const btnAdd = document.getElementById('btn-add-produto') as HTMLElement;
        btnAdd.addEventListener('click', this.addBtnHandler.bind(this));
        await this.getProducts();
    }

    private async getProducts() {
        const products: ProductsModel = await this._api.getFromApi("/produtos") as ProductsModel;
        if (products.produtos.length === 0) {
            if (this._pageContainer) {
                this._pageContainer.innerHTML += `
                <div class="mdc-typography--subtitle1 txt-empty-max-width no-product">Não há produtos cadastrados</div>
                `
            }
        } else {
            products.produtos.forEach((product) => {
                const imageSrc = `${ApiHost}/produto/image/${product.idGeladinho}`;
                if (this._pageContainer) {
                    this._pageContainer.innerHTML += `
                <div class="mdc-card mdc-card--outlined card-produto" data-product-id="${product.idGeladinho}" id="${product.idGeladinho}">
                  <div class="mdc-card__primary-action card-produto-media">
                    <div class="mdc-card__media mdc-card__media--square" style="background-image: url(${imageSrc});">
                    </div>
                    <div class="mdc-card__ripple"></div>
                  </div>
                  <div class="card-content">
                      <div class="mdc-card-wrapper__text-section">
                        <div class="mdc-card__title">${product.nomeGeladinho}</div>
                        <div class="mdc-card__subhead">R$${product.precoGeladinho.toFixed(2)}<br>${product.descricaoGeladinho}</div>
                      </div>
                      <div class="mdc-card__actions">
                        <div class="mdc-card__action-buttons card-produto-action_buttons">
                          <button class="mdc-button mdc-card__action mdc-card__action--button button-card-product__edit" data-product-id="${product.idGeladinho}">
                            <div class="mdc-button__ripple"></div>
                            <span class="mdc-button__label">Editar</span>
                          </button>
                          <button class="mdc-button mdc-card__action mdc-card__action--button button-card-product__delete" data-product-id="${product.idGeladinho}">
                            <div class="mdc-button__ripple"></div>
                            <span class="mdc-button__label">Apagar</span>
                          </button>
                        </div>
                      </div>
                    </div>
                </div>
                `;
                }
            })

            const cards = document.getElementsByClassName('card-produto');
            if (cards) {
                for (let i = 0; i < cards.length; i++) {
                    const lastCard = cards[i] as HTMLElement;
                    const lastCardMedia = lastCard.querySelector('.mdc-card__media') as HTMLElement;
                    const lastCardCt = lastCard.querySelector('.card-produto-media') as HTMLElement;
                    const lastCardButtonEdit = lastCard.querySelector('.button-card-product__edit') || new HTMLElement();
                    const lastCardButtonDelete = lastCard.querySelector('.button-card-product__delete') || new HTMLElement();
                    lastCardMedia.style.width = `${lastCardMedia.offsetHeight}px`;
                    lastCardCt.style.width = `${lastCardMedia.offsetHeight}px`;
                    lastCardMedia.style.height = `${lastCard.offsetHeight}px`;
                    lastCardCt.style.height = `${lastCard.offsetHeight}px`;

                    new MDCRipple(lastCardButtonEdit);
                    new MDCRipple(lastCardButtonDelete);
                    lastCardButtonEdit.addEventListener('click', (event) => this.editProduct(event, lastCard.id));
                    lastCardButtonDelete.addEventListener('click', (event) => this.deleteProduct(event, lastCard.id));
                }
            }
        }
    }

    private editProduct = async (event: Event, productId: string): Promise<void> => {
        if (this._dialog && this._dialogStruct) {
            this._dialogStruct.body.innerHTML = "";
            this._dialog.open();
            this._dialogStruct.header.innerText = "Editar Produto";
            this._dialogStruct.buttonOkLabel.innerText = "Salvar";
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-product-dialog-name", "Nome", TextFieldTypes.SINGLE, "text"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-product-dialog-description", "Descrição", TextFieldTypes.TEXTAREA, "text"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newSelector("select-product-dialog-state", "Estado do produto", "Estado do produto", [{name: "Ativo", value: "1"}, {name: "Inativo", value: "0"}]));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-product-dialog-price", "Preço", TextFieldTypes.SINGLE, "number", "R$"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-product-dialog-quantity", "Quantidade", TextFieldTypes.SINGLE, "number"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newFilePicker("input-product-dialog-file", "Alterar Foto", productId || undefined));
            this._dialogEls = {
                name: new MDCTextField(document.getElementById("txt-product-dialog-name") || new HTMLElement()),
                description: new MDCTextField(document.getElementById("txt-product-dialog-description") || new HTMLElement()),
                price: new MDCTextField(document.getElementById("txt-product-dialog-price") || new HTMLElement()),
                quantity: new MDCTextField(document.getElementById("txt-product-dialog-quantity") || new HTMLElement()),
                state: new MDCSelect(document.getElementById("select-product-dialog-state") || new HTMLElement()),
                file: document.getElementById("input-product-dialog-file") || new HTMLElement()
            }

            let productImageChanged = false;
            const fileInput = this._dialogEls.file.querySelector('input');

            new MDCRipple(this._dialogEls.file.querySelector('button') || new HTMLElement());

            if (fileInput) {
                fileInput.addEventListener('change', () => {
                    if (fileInput.files && fileInput.files.length > 0) {
                        productImageChanged = true;
                    }
                });
                this._dialogEls.file.querySelector('button')?.addEventListener('click', () => fileInput.click());
            }

            const productData: ProductsModel = await this._api.getFromApi("/produto", productId || "") as ProductsModel;

            const item = productData.produtos[0];

            const stocks: StockModel = await this._api.getFromApi("/estoque/get", productId) as StockModel;
            const stock: Stock = stocks.estoques[0];

            this._dialogEls.quantity.getDefaultFoundation().setValue(stock.quantidadeEstoque.toString());
            this._dialogEls.name.getDefaultFoundation().setValue(item.nomeGeladinho);
            this._dialogEls.description.getDefaultFoundation().setValue(item.descricaoGeladinho);
            this._dialogEls.price.getDefaultFoundation().setValue(Number(item.precoGeladinho).toFixed(2));

            if (item.statusGeladinho) {
                this._dialogEls.state.setValue("1")
            } else {
                this._dialogEls.state.setValue("0");
            }

            this._dialogStruct.buttonOk.addEventListener('click', async () => {
                this._pendingAction = true;
                if (this._dialogEls && this.checkProductData(this._dialogEls)) {
                    if (this._dialogStruct) {
                        this._dialogStruct.buttonOk.setAttribute("disabled", "");
                    }
                    if (productId) {
                        const productData: Products = {
                            idGeladinho: productId,
                            nomeGeladinho: this._dialogEls.name.getDefaultFoundation().getValue(),
                            descricaoGeladinho: this._dialogEls.description.getDefaultFoundation().getValue(),
                            precoGeladinho: Number(this._dialogEls.price.getDefaultFoundation().getValue()),
                            statusGeladinho: Boolean(Number(this._dialogEls.state.getDefaultFoundation().getValue()))
                        }

                        if (productImageChanged && fileInput && fileInput.files) {
                            const formData = new FormData();
                            formData.append("file", fileInput.files[0]);

                            await fetch(`${ApiHost}/produto/image/${productId}`, {
                                method: "POST",
                                body: formData
                            });
                        }

                        const stockData: Stock = {
                            idProduto: productId,
                            quantidadeEstoque: Number(this._dialogEls.quantity.getDefaultFoundation().getValue())
                        }

                        const resStock = await this._api.putToApi("/estoque/editar", stockData, productId);
                        const res = await this._api.putToApi("/produto", productData, productId);
                        this._dialog?.close();
                        if (res["error"] && resStock["error"]) {
                            this._context.showSnackbar("Erro ao Editar Produto");
                        } else {
                            this._context.showSnackbar("Produto Editado com Sucesso");
                            this.destroy();
                            this.buildPage();
                        }
                    }
                } else {
                    this._context.showSnackbar("Verifique os dados");
                }
            })
        }
    }

    private addBtnHandler = () => {
        if (this._dialog && this._dialogStruct) {
            this._dialogStruct.body.innerHTML = "";
            this._dialog.open();
            this._dialogStruct.header.innerText = "Adicionar Produto";
            this._dialogStruct.buttonOkLabel.innerText = "Salvar";
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-product-dialog-name", "Nome", TextFieldTypes.SINGLE, "text"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-product-dialog-description", "Descrição", TextFieldTypes.TEXTAREA, "text"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newSelector("select-product-dialog-state", "Estado do produto", "Estado do produto", [{name: "Ativo", value: "1"}, {name: "Inativo", value: "0"}]));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-product-dialog-price", "Preço", TextFieldTypes.SINGLE, "number", "R$"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newTextField("txt-product-dialog-quantity", "Quantidade", TextFieldTypes.SINGLE, "number"));
            this._dialogStruct.body.insertAdjacentElement('beforeend', this.newFilePicker("input-product-dialog-file", "Adicionar Foto"));
            const dialogEls: DialogEls = {
                name: new MDCTextField(document.getElementById("txt-product-dialog-name") || new HTMLElement()),
                description: new MDCTextField(document.getElementById("txt-product-dialog-description") || new HTMLElement()),
                price: new MDCTextField(document.getElementById("txt-product-dialog-price") || new HTMLElement()),
                quantity: new MDCTextField(document.getElementById("txt-product-dialog-quantity") || new HTMLElement()),
                state: new MDCSelect(document.getElementById("select-product-dialog-state") || new HTMLElement()),
                file: document.getElementById("input-product-dialog-file") || new HTMLElement()
            }

            let productImageChanged = false;
            const fileInput = dialogEls.file.querySelector('input');

            new MDCRipple(dialogEls.file.querySelector('button') || new HTMLElement());

            if (fileInput) {
                fileInput.addEventListener('change', () => {
                    if (fileInput.files && fileInput.files.length > 0) {
                        productImageChanged = true;
                    }
                });
                dialogEls.file.querySelector('button')?.addEventListener('click', () => fileInput.click());
            }

            dialogEls.state.setValue("1");

            this._dialogStruct.buttonOk.addEventListener('click', async () => {
                this._pendingAction = true;
                if (this.checkProductData(dialogEls) && productImageChanged) {
                    if (this._dialogStruct) {
                        this._dialogStruct.buttonOk.setAttribute("disabled", "");
                    }
                    const productData: Products = {
                        idGeladinho: crypto.randomUUID().toString(),
                        nomeGeladinho: dialogEls.name.getDefaultFoundation().getValue(),
                        descricaoGeladinho: dialogEls.description.getDefaultFoundation().getValue(),
                        precoGeladinho: Number(dialogEls.price.getDefaultFoundation().getValue()),
                        statusGeladinho: Boolean(Number(dialogEls.state.getDefaultFoundation().getValue()))
                    }

                    if (productImageChanged && fileInput && fileInput.files) {
                        const formData = new FormData();
                        formData.append("file", fileInput.files[0]);

                        await fetch(`${ApiHost}/produto/image/${productData.idGeladinho}`, {
                            method: "POST",
                            body: formData
                        });
                    }
                    const stockData: Stock = {
                        idProduto: productData.idGeladinho,
                        quantidadeEstoque: Number(dialogEls.quantity.getDefaultFoundation().getValue())
                    }

                    const res = await this._api.postToApi("/produto", productData);
                    const resStock = await this._api.putToApi("/estoque/editar", stockData, stockData.idProduto);
                    this._dialog?.close();
                    if (res["error"] && resStock["error"]) {
                        this._context.showSnackbar("Erro ao Adicionar Produto");
                        if (this._dialogStruct) {
                            this._dialogStruct.buttonOk.removeAttribute("disabled");
                        }
                    } else {
                        this._context.showSnackbar("Produto Adicionado com Sucesso");
                        this.destroy();
                        this.buildPage();
                    }
                } else {
                    this._context.showSnackbar("Verifique os dados");
                }
            })
        }
    }

    private deleteProduct = async (event: Event, productId: string): Promise<void> => {
        if (productId) {
            const res = await this._api.deleteFromApi("/produto", productId);
            if (res["error"]) {
                this._context.showSnackbar("Erro ao Remover Produto");
            } else {
                this.destroy();
                this.buildPage();
                this._context.showSnackbar("Produto removido com sucesso");
            }
        }
    }

    public checkProductData(dialogEls: DialogEls): boolean {
        let isValid = true;

        if (!dialogEls.name.getDefaultFoundation().getValue().trim() || dialogEls.name.getDefaultFoundation().getValue().trim() === "" || dialogEls.name.getDefaultFoundation().getValue().trim() === undefined) {
            isValid = false;
            dialogEls.name.getDefaultFoundation().setValid(false);
        }
        if (!dialogEls.description.getDefaultFoundation().getValue().trim() || dialogEls.description.getDefaultFoundation().getValue().trim() === "" || dialogEls.description.getDefaultFoundation().getValue().trim() === undefined) {
            isValid = false;
            dialogEls.description.getDefaultFoundation().setValid(false);
        }
        if (!dialogEls.quantity.getDefaultFoundation().getValue().trim() || dialogEls.quantity.getDefaultFoundation().getValue().trim() === "" || dialogEls.quantity.getDefaultFoundation().getValue().trim() === undefined) {
            isValid = false;
            dialogEls.quantity.getDefaultFoundation().setValid(false);
        }
        if (!dialogEls.price.getDefaultFoundation().getValue().trim() || dialogEls.price.getDefaultFoundation().getValue().trim() === "" || dialogEls.price.getDefaultFoundation().getValue().trim() === undefined) {
            isValid = false;
            dialogEls.price.getDefaultFoundation().setValid(false);
        }

        return isValid
    }

    private newTextField(id: string, label: string, type: TextFieldTypes, typeInput: string, prefix?: string): HTMLLabelElement {
        const newEl = document.createElement('label');
        if (type === TextFieldTypes.SINGLE) {
            newEl.className = "mdc-text-field mdc-text-field--outlined";
            newEl.id = id;
            newEl.innerHTML = `
            <span class="mdc-notched-outline">
              <span class="mdc-notched-outline__leading"></span>
              <span class="mdc-notched-outline__notch">
                <span class="mdc-floating-label" id="${id}-label">${label}</span>
              </span>
              <span class="mdc-notched-outline__trailing"></span>
            </span>
            `;
            if (prefix) {
                newEl.innerHTML += `
                <span class="mdc-text-field__affix mdc-text-field__affix--prefix">${prefix}</span>
                `;
            }
            newEl.innerHTML += `<input type="${typeInput}" class="mdc-text-field__input" aria-labelledby="${id}-label">`;
        } else if (type === TextFieldTypes.TEXTAREA) {
            newEl.className = "mdc-text-field mdc-text-field--outlined mdc-text-field--textarea mdc-text-field--no-label";
            newEl.id = id;
            newEl.innerHTML = `
            <span class="mdc-notched-outline">
              <span class="mdc-notched-outline__leading"></span>
              <span class="mdc-notched-outline__notch">
                <span class="mdc-floating-label" id="${id}-label">${label}</span>
              </span>
              <span class="mdc-notched-outline__trailing"></span>
            </span>
            <span class="mdc-text-field__resizer">
              <textarea class="mdc-text-field__input" rows="1" cols="40" aria-labelledby="${id}-label"></textarea>
            </span>
            `;
        }

        return newEl;
    }

    private newSelector(id: string, label: string, selectLabel: string, items: SelectorItems[]): HTMLDivElement {
        let listItemsHtml = ""

        for (let i=0;i<items.length;i++) {
            listItemsHtml += `
            <li class="mdc-list-item" aria-selected="false" data-value="${items[i].value}" role="option">
              <span class="mdc-list-item__ripple"></span>
              <span class="mdc-list-item__text" style="align-content: center;">
                ${items[i].name}
              </span>
            </li>
            `;
        }

        const newEl = document.createElement('div');
        newEl.id = id;
        newEl.className = "mdc-select mdc-select--outlined";
        newEl.innerHTML = `
        <div class="mdc-select__anchor" aria-labelledby="${id}-select-label">
          <span class="mdc-notched-outline">
            <span class="mdc-notched-outline__leading"></span>
            <span class="mdc-notched-outline__notch">
              <span id="${id}-select-label" class="mdc-floating-label">${selectLabel}</span>
            </span>
            <span class="mdc-notched-outline__trailing"></span>
          </span>
          <span class="mdc-select__selected-text-container">
            <span id="demo-selected-text" class="mdc-select__selected-text"></span>
          </span>
          <span class="mdc-select__dropdown-icon">
            <svg
                class="mdc-select__dropdown-icon-graphic"
                viewBox="7 10 10 5" focusable="false">
              <polygon
                  class="mdc-select__dropdown-icon-inactive"
                  stroke="none"
                  fill-rule="evenodd"
                  points="7 10 12 15 17 10">
              </polygon>
              <polygon
                  class="mdc-select__dropdown-icon-active"
                  stroke="none"
                  fill-rule="evenodd"
                  points="7 15 12 10 17 15">
              </polygon>
            </svg>
          </span>
        </div>
        
        <div class="mdc-select__menu mdc-menu mdc-menu-surface mdc-menu-surface--fullwidth" style="z-index: 10000;">
        <ul class="mdc-list" role="listbox" aria-label="${label}" id="${id}-list">${listItemsHtml}</ul></div>
        `;

        return newEl
    }

    private newFilePicker(id: string, label: string, imageId?: string): HTMLDivElement {
        const newEl = document.createElement('div');
        newEl.className = "produto-file-picker-div"
        if (imageId) {
            newEl.id = id;
            newEl.innerHTML = `
            <img class="img-dialog-product" src="${ApiHost}/produto/image/${imageId}" alt="Imagem">
            <button class="mdc-button mdc-button--outlined">
              <span class="mdc-button__ripple"></span>
              <span class="mdc-button__label">${label}</span>
            </button>
            <input type="file" style="display: none;" accept="image/*" name="image" id="${id}-input">
            `
        } else {
            newEl.id = id;
            newEl.innerHTML = `
            <button class="mdc-button mdc-button--outlined">
              <span class="mdc-button__ripple"></span>
              <span class="mdc-button__label">${label}</span>
            </button>
            <input type="file" style="display: none;" accept="image/*" name="image" id="${id}-input">
            `
        }
        return newEl
    }
}

interface DialogStruct {
    header: HTMLElement;
    body: HTMLElement;
    buttonOk: HTMLElement;
    buttonOkLabel: HTMLElement;
}

interface DialogEls {
    name: MDCTextField,
    description: MDCTextField,
    price: MDCTextField,
    quantity: MDCTextField,
    state: MDCSelect,
    file: HTMLElement
}

interface FileInput {
    root: HTMLElement,
    button: HTMLElement,
    file: HTMLInputElement
}

interface SelectorItems {
    name: string,
    value: string
}

enum TextFieldTypes {
    TEXTAREA,
    SINGLE
}