import {MDCTextField} from "@material/textfield";
import {Api} from "@/js/admin/Api";
import {MDCRipple} from "@material/ripple";
import {DeliveryFeesModel} from "@/js/declarations";
import {AdminPages} from "@/js/admin/admin_pages";

export class PageDeliveryFee {
    private _appContainer: HTMLElement;
    private _inputTaxa?: MDCTextField;
    private _buttonSave?: HTMLElement;
    private _tableCellTaxa?: HTMLElement;
    private _api: Api;
    private _id?: number;
    private _context: AdminPages;

    constructor(appContainer: HTMLElement, adminPages: AdminPages) {
        this._appContainer = appContainer;
        this._api = new Api();
        this._context = adminPages;
        this._context.setTitle("Taxa de Entrega")
        this.buildPage();
    }

    public destroy() {
        this._buttonSave?.removeEventListener('click', this.saveFee)
    }

    private async buildPage() {
        const htmlData = await fetch("/template_taxa");
        const htmlDataBody = await htmlData.text();

        this._appContainer.innerHTML = htmlDataBody;
        this._inputTaxa = new MDCTextField(document.getElementById('taxa-input') || new HTMLElement());
        this._buttonSave = document.getElementById('button-taxa') || new HTMLElement();
        this._tableCellTaxa = document.getElementById('cell-valor-atual') || new HTMLElement();
        new MDCRipple(this._buttonSave);

        const taxaAtual: DeliveryFeesModel = await this._api.getFromApi("/deliveryFee/listar") as DeliveryFeesModel;
        if (taxaAtual.deliveryFees.length > 0) {
            this._id = taxaAtual.deliveryFees[0].id;
            console.log(this._id);
            this._tableCellTaxa.innerText = `R$${Number(taxaAtual.deliveryFees[0].deliveryFee).toFixed(2)}`;
        } else {
            const dataTable = document.querySelector('.mdc-data-table') || new HTMLElement();
            dataTable.remove();
        }

        if (this._buttonSave) {
            this._buttonSave.addEventListener('click', this.saveFee);
        }
    }

    private saveFee = async (event: Event) => {
        const target = event.target as HTMLElement;

        if (this._inputTaxa && Number(this._inputTaxa.getDefaultFoundation().getValue()) > 0) {
            const newTaxa = {
                deliveryFee: Number(this._inputTaxa.getDefaultFoundation().getValue())
            }

            if (this._id) {
                const res = await this._api.putToApi("/deliveryFee/editar", newTaxa, String(this._id));
                if (res["error"]) {
                    this._context.showSnackbar("Erro ao registrar taxa")
                } else {
                    this._context.showSnackbar("Taxa registrada com sucesso")
                }
            } else {
                const res = await this._api.postToApi("/deliveryFee/registrar", newTaxa);
                if (res["error"]) {
                    this._context.showSnackbar("Erro ao registrar taxa")
                } else {
                    this._context.showSnackbar("Taxa registrada com sucesso")
                }
            }
            this._context.reload();
        } else {
            if (Number(this._inputTaxa?.getDefaultFoundation().getValue()) <= 0) {
                this._context.showSnackbar("Insira um valor maior que 0");
            }
        }
    }
}