import {MDCTextField} from "@material/textfield";
import {Api} from "@/js/admin/Api";
import {MDCRipple} from "@material/ripple";
import {DeliveryFeesModel, Settings, SettingsModel} from "@/js/declarations";
import {AdminPages} from "@/js/admin/admin_pages";
import {ApiHost} from "@/js/vars";

export class PageSettings {
    private _appContainer: HTMLElement;
    private _settingsEls?: SettingsElements;
    private _api: Api;
    private _id?: number;
    private _context: AdminPages;
    private _productImageChanged: boolean = false;

    constructor(appContainer: HTMLElement, adminPages: AdminPages) {
        this._appContainer = appContainer;
        this._api = new Api();
        this._context = adminPages;
        this._context.setTitle("Configurações")
        this.buildPage();
    }

    public destroy() {

    }

    private async buildPage() {
        const htmlData = await fetch("/template_admin_settings");
        const htmlDataBody = await htmlData.text();

        const imagePicker = this.newFilePicker("site-image-input", "Alterar Foto do Cabeçalho");

        this._appContainer.innerHTML = htmlDataBody;

        const siteSettingsCt = document.getElementById("site-settings-ct");

        siteSettingsCt ? siteSettingsCt.insertAdjacentElement('beforeend', imagePicker) : null;

        this._settingsEls = {
            emailInput: new MDCTextField(document.querySelector('#email-input') || new HTMLElement),
            smtpHostInput: new MDCTextField(document.querySelector('#smtp-domain-input') || new HTMLElement),
            smtpUsernameInput: new MDCTextField(document.querySelector('#smtp-username-input') || new HTMLElement),
            smtpPasswordInput: new MDCTextField(document.querySelector('#smtp-password-input') || new HTMLElement),
            smtpPortInput: new MDCTextField(document.querySelector('#smtp-port-input') || new HTMLElement),
            pixChave: new MDCTextField(document.querySelector('#pix-input') || new HTMLElement),
            pixNome: new MDCTextField(document.querySelector('#pix-name-input') || new HTMLElement),
            pixCidade: new MDCTextField(document.querySelector('#pix-city-input') || new HTMLElement),
            socialWhatsApp: new MDCTextField(document.querySelector('#social-whatsapp-input') || new HTMLElement),
            socialInstagram: new MDCTextField(document.querySelector('#social-instagram-input') || new HTMLElement),
            supportEmail: new MDCTextField(document.querySelector('#support-email-input') || new HTMLElement),
            supportPhone: new MDCTextField(document.querySelector('#support-phone-input') || new HTMLElement),
            siteName: new MDCTextField(document.querySelector('#site-name-input') || new HTMLElement),
            siteHeader: new MDCTextField(document.querySelector('#site-header-input') || new HTMLElement),
            siteDescription: new MDCTextField(document.querySelector('#site-description-input') || new HTMLElement),
            siteImage: document.querySelector('#site-image-input') || new HTMLElement,
            saveButton: document.querySelector('#button-save') || new HTMLElement,
        }

        this._settingsEls.socialWhatsApp.root.addEventListener('input',  (event) => {
            let input = event.target as HTMLInputElement;
            input.value = this.formatPhoneNumber(input.value);
        });

        this._settingsEls.supportPhone.root.addEventListener('input',  (event) => {
            let input = event.target as HTMLInputElement;
            input.value = this.formatPhoneNumber(input.value);
        });

        new MDCRipple(this._settingsEls.saveButton);
        const buttonImage = this._settingsEls.siteImage.querySelector('.mdc-button');
        if (buttonImage) {
            new MDCRipple(buttonImage);
            const fileInput = this._settingsEls.siteImage.querySelector('input');

            if (fileInput) {
                fileInput.addEventListener('change', () => {
                    if (fileInput.files && fileInput.files.length > 0) {
                        this._productImageChanged = true;
                    }
                });
                this._settingsEls.siteImage.querySelector('button')?.addEventListener('click', () => fileInput.click());
            }
        }

        let settings: Settings[] = await this.getSettings();
        await this.loadSettings(settings);

        this._settingsEls.saveButton.addEventListener('click',() => this.saveSettings());
    }

    private async getSettings(): Promise<Settings[]> {
        let settings: Settings[] = [];

        let email: SettingsModel = await this._api.getFromApi("/settings/email");
        if (email && !email.error) {settings.push(email.setting)}
        let smtpHost: SettingsModel = await this._api.getFromApi("/settings/smtp/domain");
        if (smtpHost && !smtpHost.error) {settings.push(smtpHost.setting)}
        let smtpPort: SettingsModel = await this._api.getFromApi("/settings/smtp/port");
        if (smtpPort && !smtpPort.error) {settings.push(smtpPort.setting)}
        let smtpUsername: SettingsModel = await this._api.getFromApi("/settings/smtp/username");
        if (smtpUsername && !smtpUsername.error) {settings.push(smtpUsername.setting)}
        let pixChave: SettingsModel = await this._api.getFromApi("/settings/pix/chave");
        if (pixChave && !pixChave.error) {settings.push(pixChave.setting)}
        let pixNome: SettingsModel = await this._api.getFromApi("/settings/pix/nome");
        if (pixNome && !pixNome.error) {settings.push(pixNome.setting)}
        let pixCidade: SettingsModel = await this._api.getFromApi("/settings/pix/cidade");
        if (pixCidade && !pixCidade.error) {settings.push(pixCidade.setting)}
        let socialWhatsApp: SettingsModel = await this._api.getFromApi("/settings/social/whatsapp");
        if (socialWhatsApp &&!socialWhatsApp.error) {settings.push(socialWhatsApp.setting)}
        let socialInstagram: SettingsModel = await this._api.getFromApi("/settings/social/instagram");
        if (socialInstagram &&!socialInstagram.error) {settings.push(socialInstagram.setting)}
        let supportEmail: SettingsModel = await this._api.getFromApi("/settings/support/email");
        if (supportEmail &&!supportEmail.error) {settings.push(supportEmail.setting)}
        let supportPhone: SettingsModel = await this._api.getFromApi("/settings/support/phone");
        if (supportPhone &&!supportPhone.error) {settings.push(supportPhone.setting)}
        let siteName: SettingsModel = await this._api.getFromApi("/settings/site/name");
        if (siteName &&!siteName.error) {settings.push(siteName.setting)}
        let siteHeader: SettingsModel = await this._api.getFromApi("/settings/site/header");
        if (siteHeader &&!siteHeader.error) {settings.push(siteHeader.setting)}
        let siteDescription: SettingsModel = await this._api.getFromApi("/settings/site/description");
        if (siteDescription &&!siteDescription.error) {settings.push(siteDescription.setting)}

        return settings;
    }

    private async loadSettings(settings: Settings[]) {
        if (settings.length > 0 && this._settingsEls) {
            for (const setting of settings) {
                switch (setting.key) {
                    case "email":
                        this._settingsEls.emailInput.value = setting.value;
                        break;
                    case "smtp_domain":
                        this._settingsEls.smtpHostInput.value = setting.value;
                        break;
                    case "smtp_username":
                        this._settingsEls.smtpUsernameInput.value = setting.value;
                        break;
                    case "smtp_password":
                        this._settingsEls.smtpPasswordInput.value = setting.value;
                        break;
                    case "smtp_port":
                        this._settingsEls.smtpPortInput.value = setting.value;
                        break;
                    case "pix_chave":
                        this._settingsEls.pixChave.value = setting.value;
                        break;
                    case "pix_nome":
                        this._settingsEls.pixNome.value = setting.value;
                        break;
                    case "pix_cidade":
                        this._settingsEls.pixCidade.value = setting.value;
                        break;
                    case "social_whatsapp":
                        this._settingsEls.socialWhatsApp.value = setting.value;
                        break;
                    case "social_instagram":
                        this._settingsEls.socialInstagram.value = setting.value;
                        break;
                    case "support_email":
                        this._settingsEls.supportEmail.value = setting.value;
                        break;
                    case "support_phone":
                        this._settingsEls.supportPhone.value = setting.value;
                        break;
                    case "site_name":
                        this._settingsEls.siteName.value = setting.value;
                        break;
                    case "site_header":
                        this._settingsEls.siteHeader.value = setting.value;
                        break;
                    case "site_description":
                        this._settingsEls.siteDescription.value = setting.value;
                        break;
                }
            }
        }
    }

    private async saveSettings() {
        if (this._settingsEls) {
            let settingsData: SettingsData = {
                email: {
                    key: "email",
                    value: this._settingsEls.emailInput.value.trim()
                },
                smtpHost: {
                    key: "smtp_domain",
                    value: this._settingsEls.smtpHostInput.value.trim()
                },
                smtpUsername: {
                    key: "smtp_username",
                    value: this._settingsEls.smtpUsernameInput.value.trim()
                },
                smtpPassword: {
                    key: "smtp_password",
                    value: this._settingsEls.smtpPasswordInput.value.trim()
                },
                smtpPort: {
                    key: "smtp_port",
                    value: this._settingsEls.smtpPortInput.value.trim()
                },
                pixChave: {
                    key: "pix_chave",
                    value: this._settingsEls.pixChave.value.trim()
                },
                pixNome: {
                    key: "pix_nome",
                    value: this._settingsEls.pixNome.value.trim()
                },
                pixCidade: {
                    key: "pix_cidade",
                    value: this._settingsEls.pixCidade.value.trim()
                },
                socialWhatsApp: {
                    key: "social_whatsapp",
                    value: this.stripPhoneNumberFormatting(this._settingsEls.socialWhatsApp.value.trim())
                },
                socialInstagram: {
                    key: "social_instagram",
                    value: this._settingsEls.socialInstagram.value.trim()
                },
                supportEmail: {
                    key: "support_email",
                    value: this._settingsEls.supportEmail.value.trim()
                },
                supportPhone: {
                    key: "support_phone",
                    value: this.stripPhoneNumberFormatting(this._settingsEls.supportPhone.value.trim())
                },
                siteName: {
                    key: "site_name",
                    value: this._settingsEls.siteName.value.trim()
                },
                siteHeader: {
                    key: "site_header",
                    value: this._settingsEls.siteHeader.value.trim()
                },
                siteDescription: {
                    key: "site_description",
                    value: this._settingsEls.siteDescription.value.trim()
                }
            };

            await this._api.postToApi("/settings/email", settingsData.email);
            await this._api.postToApi("/settings/smtp/domain", settingsData.smtpHost);
            await this._api.postToApi("/settings/smtp/username", settingsData.smtpUsername);
            if (settingsData.smtpPassword.value.trim()) {await this._api.postToApi("/settings/smtp/password", settingsData.smtpPassword);}
            await this._api.postToApi("/settings/smtp/port", settingsData.smtpPort);
            await this._api.postToApi("/settings/pix/chave", settingsData.pixChave);
            await this._api.postToApi("/settings/pix/nome", settingsData.pixNome);
            await this._api.postToApi("/settings/pix/cidade", settingsData.pixCidade);
            await this._api.postToApi("/settings/social/whatsapp", settingsData.socialWhatsApp);
            await this._api.postToApi("/settings/social/instagram", settingsData.socialInstagram);
            await this._api.postToApi("/settings/support/email", settingsData.supportEmail);
            await this._api.postToApi("/settings/support/phone", settingsData.supportPhone);
            await this._api.postToApi("/settings/site/name", settingsData.siteName);
            await this._api.postToApi("/settings/site/header", settingsData.siteHeader);
            await this._api.postToApi("/settings/site/description", settingsData.siteDescription);
            const fileInput = this._settingsEls.siteImage.querySelector('input');
            if (this._productImageChanged && fileInput && fileInput.files) {
                const formData = new FormData();
                formData.append("file", fileInput.files[0]);

                await fetch(`${ApiHost}/settings/site/image`, {
                    method: "POST",
                    body: formData
                });
            }
            this._context.showSnackbar("Configurações Salvas com Sucesso");
        }
    }

    private formatPhoneNumber(value: string): string {
        if (!value) return value;
        const phoneNumber = value.replace(/[^\d]/g, '');
        const phoneNumberLength = phoneNumber.length;
        if (phoneNumberLength < 4) return phoneNumber;
        if (phoneNumberLength < 7) {
            return `(${phoneNumber.slice(0, 2)}) ${phoneNumber.slice(2)}`;
        }
        return `(${phoneNumber.slice(0, 2)}) ${phoneNumber.slice(2, 7)}-${phoneNumber.slice(7, 11)}`;
    }

    private stripPhoneNumberFormatting(value: string): string {
        return value.replace(/[^\d]/g, '');
    }

    private newFilePicker(id: string, label: string): HTMLElement {
        const newEl = document.createElement('div');
        newEl.className = "produto-file-picker-div"
        newEl.id = id;
        newEl.innerHTML = `
            <button class="mdc-button mdc-button--outlined">
              <span class="mdc-button__ripple"></span>
              <span class="mdc-button__label">${label}</span>
            </button>
            <input type="file" style="display: none;" accept="image/*" name="image" id="${id}-input">
            `
        return newEl as HTMLElement
    }
}

interface SettingsElements {
    emailInput: MDCTextField,
    smtpUsernameInput: MDCTextField,
    smtpPasswordInput: MDCTextField,
    smtpHostInput: MDCTextField,
    smtpPortInput: MDCTextField,
    pixChave: MDCTextField,
    pixNome: MDCTextField,
    pixCidade: MDCTextField,
    socialWhatsApp: MDCTextField,
    socialInstagram: MDCTextField,
    supportEmail: MDCTextField,
    supportPhone: MDCTextField,
    siteName: MDCTextField,
    siteHeader: MDCTextField,
    siteDescription: MDCTextField,
    siteImage: HTMLElement,
    saveButton: HTMLElement,
}

interface SettingsData {
    email: Settings,
    smtpHost: Settings,
    smtpUsername: Settings,
    smtpPassword: Settings,
    smtpPort: Settings,
    pixChave: Settings,
    pixNome: Settings,
    pixCidade: Settings,
    socialWhatsApp: Settings,
    socialInstagram: Settings,
    supportEmail: Settings,
    supportPhone: Settings,
    siteName: Settings,
    siteHeader: Settings,
    siteDescription: Settings,
}

interface SelectorItems {
    name: string,
    value: string
}

enum TextFieldTypes {
    TEXTAREA,
    SINGLE
}