import {ApiHost} from "@/js/vars";
import moment from "moment";
import {MDCSnackbar} from '@material/snackbar';

const snackbar = new MDCSnackbar(document.querySelector('.mdc-snackbar'));

document.addEventListener("DOMContentLoaded", function() {
    inicializarAvaliacoesEEstatisticas();

    document.querySelector('#arrow--right').addEventListener('click', moverSlider);
    document.querySelector('#arrow--left').addEventListener('click', moverSlider);
    document.getElementById('add-review-btn').addEventListener('click', abrirFormulario);
    document.getElementById('submit-review').addEventListener('click', enviarAvaliacao);
    document.getElementById('close-btn').addEventListener('click', fecharFormulario);
});

let indiceSlide = 0;

function carregarEstrelas(estrelas) {
    const estrelaCheia = `<i class="fas fa-star" style="color: gold;"></i>`;
    const meiaEstrela = `<i class="fas fa-star-half-alt" style="color: gold;"></i>`;
    const estrelaVazia = `<i class="far fa-star" style="color: gold;"></i>`;

    let htmlEstrelas = '';
    for (let i = 0; i < Math.floor(estrelas); i++) {
        htmlEstrelas += estrelaCheia;
    }
    if (estrelas % 1 !== 0) {
        htmlEstrelas += meiaEstrela;
    }
    const estrelasRestantes = 5 - Math.ceil(estrelas);
    for (let i = 0; i < estrelasRestantes; i++) {
        htmlEstrelas += estrelaVazia;
    }
    return htmlEstrelas;
}

function carregarAvaliacoes(avaliacao) {
    return `
    <div class="review">
      <p class="review__name"><strong>${avaliacao.name}</strong></p>
      <p class="review__rating"><strong>Avaliação do Cliente: ${carregarEstrelas(avaliacao.rating)}</strong></p>
      <p class="review__comment">${avaliacao.texto}</p>
      <p class="review__date">Data da Avaliação: ${moment.unix(avaliacao.date).format('DD-MM-YYYY HH:mm:ss')}</p>
    </div>
  `;
}

function moverSlider(e) {
    const containerAvaliacoes = document.querySelector('.reviews');
    const avaliacoes = Array.from(containerAvaliacoes.children);

    if (e.currentTarget.id === 'arrow--right') {
        indiceSlide = (indiceSlide + 1) % avaliacoes.length;
    } else {
        indiceSlide = (indiceSlide - 1 + avaliacoes.length) % avaliacoes.length;
    }

    containerAvaliacoes.style.transform = `translateX(${-100 * indiceSlide}%)`;
}

function abrirFormulario() {
    document.getElementById('modal').style.display = 'block';
}

async function enviarAvaliacao() {
    const estrelas = parseInt(document.getElementById('stars').value);
    const nome = document.getElementById('name').value;
    const comentario = document.getElementById('comment').value;
    const idCliente = document.getElementById('idCliente').value;
    const publico = document.getElementById('publico').checked;

    if (estrelas && comentario && estrelas >= 1 && estrelas <= 5) {
        const novaAvaliacao = {
            clientId: idCliente,
            rating: estrelas,
            texto: comentario,
            publico: publico,
            name: nome,
            date: moment().unix()
        };

        fecharFormulario();

        const response = await fetch(`${ApiHost}/feedback`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(novaAvaliacao),
        });

        let avaliacaoAdicionada = await response.json();
        avaliacaoAdicionada = avaliacaoAdicionada["feedbackList"];
        document.querySelector('.reviews').innerHTML += carregarAvaliacoes(avaliacaoAdicionada);

        document.getElementById('modal').style.display = 'none';
        document.getElementById('name').value = '';
        document.getElementById('comment').value = '';

        await inicializarAvaliacoesEEstatisticas();
    } else {
        snackbar.open();
        snackbar.labelText = "Certifique-se de que todos os campos estão preenchidos";
    }
}

function fecharFormulario() {
    document.getElementById('modal').style.display = 'none';
    document.getElementById('stars').value = '';
}

async function inicializarAvaliacoesEEstatisticas() {
    const resposta = await fetch(`${ApiHost}/feedback`);
    let avaliacoes = await resposta.json();

    avaliacoes = avaliacoes["feedbackList"];

    mostrarEstatisticas(avaliacoes);
    document.querySelector('.reviews').innerHTML = avaliacoes.map(avaliacao => carregarAvaliacoes(avaliacao)).join('');
}

function mostrarEstatisticas(avaliacoes) {
    const totalEstrelas = avaliacoes.reduce((acc, curr) => acc + curr.rating, 0);
    const mediaAvaliacoes = avaliacoes.length ? (totalEstrelas / avaliacoes.length) : 0;
    document.getElementById('average-rating').textContent = mediaAvaliacoes.toFixed(1);
    document.getElementById('total-reviews').textContent = avaliacoes.length;
}
