import {ApiHost, PagesD} from "@/js/vars";
import {MDCSnackbar} from '@material/snackbar';
import {MDCDialog} from '@material/dialog';
import {CartDialog, Customer} from "@/js/cart-dialog";
import {Api} from "@/js/admin/Api";
import {MDCRipple} from "@material/ripple";
const dialog = new MDCDialog(document.querySelector('.mdc-dialog'));

const snackbar = new MDCSnackbar(document.querySelector('.mdc-snackbar'));
var carrinhoVisivel = false;

const api = new Api();

if (document.readyState == 'loading') {
    document.addEventListener('DOMContentLoaded', pronto)
} else {
    const container = document.querySelector('.container-items');

    fetch(`${ApiHost}/produtos`)
            .then(response => response.json())
            .then(produtos => {
                for (let i=0;i<produtos["produtos"].length;i++){
                    if (!produtos["produtos"][i].statusGeladinho) {
                        produtos["produtos"].splice(i, 1);
                    }
                }
                if (produtos["produtos"].length === 0) {
                    const item = document.createElement('div');
                    item.style.width = '100%';
                    item.style.textAlign = 'center';
                    item.className = "mdc-typography--headline5";
                    item.innerHTML = "Ainda não há produtos cadastrados, por favor, volte mais tarde!";
                    container.parentElement.appendChild(item);
                    document.querySelector('#carrinho').remove();
                    container.remove();
                } else {
                    produtos["produtos"].forEach(produto => {
                        const item = document.createElement('div');
                        item.setAttribute('data-item-id', produto.idGeladinho);
                        const imageSrc = `${ApiHost}/produto/image/${produto.idGeladinho}`;
                        item.innerHTML = `
                    <div class="mdc-card mdc-card--outlined">
                    <div class="mdc-card__primary-action card-produto-media">
                    <div class="mdc-card__media mdc-card__media--square" style="background-image: url(${imageSrc});">
                    </div>
                    <div class="mdc-card__ripple"></div>
                  </div>
                  <div class="card-content">
                      <div class="mdc-card-wrapper__text-section">
                        <div class="mdc-card__title-container">
                            <div class="mdc-card__title">${produto.nomeGeladinho}</div>
                            <div class="mdc-card__title-right mdc-typography--subtitle1">R$${produto.precoGeladinho.toFixed(2)}</div>
                        </div>
                        <div class="mdc-card__subhead">${produto.descricaoGeladinho}</div>
                      </div>
                      <div class="mdc-card__actions">
                        <div class="mdc-card__action-buttons card-produto-action_buttons">
                          <button class="mdc-button mdc-card__action mdc-card__action--button button-card-product__add" id="${produto.idGeladinho}">
                            <div class="mdc-button__ripple"></div>
                            <span class="mdc-button__label">Adicionar</span>
                          </button>
                        </div>
                      </div>
                    </div>
                    </div>
                    `

                        container.appendChild(item);
                    });
                }
                pronto();
            })
            .catch(error => console.error('Erro ao listar produtos:', error));
}

function pronto() {
    
    // Eliminar
    var botoesEliminarItem = document.getElementsByClassName('btn-eliminar');
    for (var i = 0; i < botoesEliminarItem.length; i++) {
        var botao = botoesEliminarItem[i];
        botao.addEventListener('click', eliminarItemCarrinho);
    }

    // Aumentar quantidade
    var botoesAumentarQuantidade = document.getElementsByClassName('aumentar-quantidade');
    for (var i = 0; i < botoesAumentarQuantidade.length; i++) {
        var botao = botoesAumentarQuantidade[i];
        botao.addEventListener('click', aumentarQuantidade);
    }

    // Diminuir quantidade
    var botoesDiminuirQuantidade = document.getElementsByClassName('diminuir-quantidade');
    for (var i = 0; i < botoesDiminuirQuantidade.length; i++) {
        var botao = botoesDiminuirQuantidade[i];
        botao.addEventListener('click', diminuirQuantidade);
    }

    // Adicionar ao carrinho
    var botoesAdicionarAoCarrinho = document.getElementsByClassName('button-card-product__add');
    for (var i = 0; i < botoesAdicionarAoCarrinho.length; i++) {
        const botao = botoesAdicionarAoCarrinho[i];
        new MDCRipple(botao);
        let id = botao.id;
        (function(id) {
            botao.addEventListener('click', (event) => adicionarAoCarrinhoClicado(event, id));
        })(id);
    }

    // Comprar
    document.getElementsByClassName('btn-pagar')[0].addEventListener('click', iniciarCompra);
}

function iniciarCompra() {
    dialog.open();
}

function pagarClicado(clienteId) {
    let itensCarrinho = document.querySelectorAll('.carrinho-item');
    
    let itens = [];

    console.log(itensCarrinho);

    for (let i=0; i<itensCarrinho.length; i++) {
        const newItem = {
            idProduto: itensCarrinho[i].getAttribute('data-item-id'),
            quantidade: itensCarrinho[i].querySelector('.carrinho-item-quantidade').value
        }

        itens.push(newItem);
    }

    submitOrder(itens, clienteId);

    atualizarTotalCarrinho();
    ocultarCarrinho();
}

function submitOrder(items, idCliente) {
    const order_payload = {
        pedidoId: crypto.randomUUID(),
        idCliente: idCliente,
        geladinhos: items,
    }

    fetch(`${ApiHost}/pedido/new`, {
        "method": "POST",
        "body": JSON.stringify(order_payload),
        "headers": {
            "Content-Type": "application/json"
        }
    }).then(res => res.json()).then(async (res) => {
        dialog.close();
        try {
            if(res["error"]) {
                snackbar.open();
                snackbar.labelText = "Erro ao realizar pedido"
                console.log("res received")
            } else {
                snackbar.open();
                snackbar.labelText = "Pedido realizado com sucesso!"
                window.location.href = `${PagesD.order_confirmation}?order=${res["message"]}`
            }
        } catch (error) {
            snackbar.open();
            snackbar.labelText = "Erro ao realizar pedido"
            console.log("res received")
        }
    })
}

async function adicionarAoCarrinhoClicado(evento, idProduct) {
    let res = await fetch(`${ApiHost}/produto/${idProduct}`).then((res) => res.json());

    const productData = res["produtos"][0];
    const titulo = productData.nomeGeladinho;
    const preco = productData.precoGeladinho;
    const srcImagem = `${ApiHost}/produto/image/${productData.idGeladinho}`;

    adicionarItemAoCarrinho(titulo, preco, srcImagem, idProduct);

    tornarCarrinhoVisivel();
}

function tornarCarrinhoVisivel() {
    carrinhoVisivel = true;
    var carrinho = document.getElementsByClassName('carrinho')[0];
    carrinho.style.marginRight = '0';
    carrinho.style.opacity = '1';

    var itens = document.getElementsByClassName('container-itens')[0];
    //itens.style.width = '60%';
}

function adicionarItemAoCarrinho(titulo, preco, srcImagem, idProduto) {
    let item = document.createElement('div');
    item.classList.add('item');
    let itensCarrinho = document.getElementsByClassName('carrinho-items');
    let itemCarrinho = document.getElementsByClassName('carrinho-item');

    for (let i = 0; i < itemCarrinho.length; i++) {
        if (itemCarrinho[i].getAttribute("data-item-id") === idProduto) {
            snackbar.open();
            snackbar.labelText = "O item já foi adicionado ao carrinho"
            return;
        }
    }

    let conteudoItemCarrinho = `
        <div class="carrinho-item" data-item-id="${idProduto}">
            <img src="${srcImagem}" width="80px" alt="">
            <div class="carrinho-item-detalhes">
                <span class="carrinho-item-titulo" id="cart-item-title">${titulo}</span>
                <div class="seletor-quantidade">
                    <i class="fa-solid fa-minus diminuir-quantidade"></i>
                    <input type="text" value="1" class="carrinho-item-quantidade" id="cart-item-qty" disabled>
                    <i class="fa-solid fa-plus aumentar-quantidade"></i>
                </div>
                <span class="carrinho-item-preco" data-value="${preco}">R$${preco}</span>
            </div>
            <button class="btn-eliminar">
                <i class="fa-solid fa-trash"></i>
            </button>
        </div>
    `;
    item.innerHTML = conteudoItemCarrinho;
    itensCarrinho[0].append(item);


    item.getElementsByClassName('btn-eliminar')[0].addEventListener('click', eliminarItemCarrinho);

    let botaoDiminuirQuantidade = item.getElementsByClassName('diminuir-quantidade')[0];
    botaoDiminuirQuantidade.addEventListener('click', diminuirQuantidade);


    let botaoAumentarQuantidade = item.getElementsByClassName('aumentar-quantidade')[0];
    botaoAumentarQuantidade.addEventListener('click', aumentarQuantidade);

    atualizarTotalCarrinho();
}

function aumentarQuantidade(evento) {
    var botaoClicado = evento.target;
    var seletor = botaoClicado.parentElement;
    var quantidadeAtual = seletor.getElementsByClassName('carrinho-item-quantidade')[0].value;
    quantidadeAtual++;
    seletor.getElementsByClassName('carrinho-item-quantidade')[0].value = quantidadeAtual;
    atualizarTotalCarrinho();
}

function diminuirQuantidade(evento) {
    var botaoClicado = evento.target;
    var seletor = botaoClicado.parentElement;
    var quantidadeAtual = seletor.getElementsByClassName('carrinho-item-quantidade')[0].value;
    quantidadeAtual--;
    if (quantidadeAtual >= 1) {
        seletor.getElementsByClassName('carrinho-item-quantidade')[0].value = quantidadeAtual;
        atualizarTotalCarrinho();
    }
}

function eliminarItemCarrinho(evento) {
    var botaoClicado = evento.target;
    botaoClicado.parentElement.parentElement.remove();
    atualizarTotalCarrinho();
    ocultarCarrinho();
}

function ocultarCarrinho() {
    var itensCarrinho = document.getElementsByClassName('carrinho-items')[0];
    if (itensCarrinho.childElementCount == 0) {
        var carrinho = document.getElementsByClassName('carrinho')[0];
        carrinho.style.marginRight = '-100%';
        carrinho.style.opacity = '0';
        carrinhoVisivel = false;

        var itens = document.getElementsByClassName('container-itens')[0];
        itens.style.width = '100%';
    }
}

async function atualizarTotalCarrinho() {

    var containerCarrinho = document.getElementsByClassName('carrinho')[0];
    var itensCarrinho = containerCarrinho.getElementsByClassName('carrinho-item');
    var total = 0;

    for (var i = 0; i < itensCarrinho.length; i++) {
        var item = itensCarrinho[i];
        var precoElemento = item.getElementsByClassName('carrinho-item-preco')[0];
        var preco = parseFloat(precoElemento.getAttribute('data-value'));
        var quantidadeItem = item.getElementsByClassName('carrinho-item-quantidade')[0].value;
        total = total + (preco * quantidadeItem);
    }


    const taxaRes = await fetch(`${ApiHost}/deliveryFee/listar`);
    const taxaResParsed = await taxaRes.json();

    document.getElementById('txt-taxa-de-entrega').innerText = `R$${taxaResParsed["deliveryFees"][0]["deliveryFee"].toFixed(2)}`

    total += Number(taxaResParsed["deliveryFees"][0]["deliveryFee"]);
    total = Math.round(total * 100) / 100;
    document.getElementsByClassName('carrinho-preco-total')[0].innerText = 'R$' + total.toLocaleString("pt-BR") + ",00";

}

let dialogLayout;

dialog.listen('MDCDialog:opened', async () => {
    dialogLayout = new CartDialog();
    dialogLayout.btnSend.addEventListener('click', checkOutCheck);
});

dialog.listen('MDCDialog:closing', function() {
    dialogLayout.btnSend.removeEventListener('click', checkOutCheck);
    dialogLayout = null;
});

async function checkOutCheck() {
    if (dialogLayout.checkCustomerData()) {
        const customer = new Customer(dialogLayout.getCustomerData());
        const existing = customer.checkCustomer();
        if (existing) {
            if (await customer.updateCustomer(existing)) {
                pagarClicado(existing.idCliente);
            } else {
                snackbar.open();
                snackbar.labelText = "Erro ao atualizar dados do cliente";
            }
        } else {
            if (await customer.addNewCustomer()) {
                pagarClicado(customer.customerDataLocal.idCliente);
            } else {
                snackbar.open();
                snackbar.labelText = "Erro ao adicionar novo cliente";
            }
        }
    } else {
        snackbar.open();
        snackbar.labelText = "Verifique se todos os dados estão preenchidos";
    }
}