import {MDCTextField} from '@material/textfield';
import {MDCRipple} from '@material/ripple';
import {ApiHost} from "@/js/vars";
import * as console from "node:console";

export interface CustomerData {
    nome: string,
    endereco: string,
    numeroTelefone: string,
    numeroCasa: string,
    referenciaCasa: string,
    email: string
}

export interface CustomerDataLocal {
    idCliente: string,
    nome: string,
    endereco: string,
    numeroTelefone: string,
    numeroCasa: string,
    referenciaCasa: string,
    email: string
}

export class CartDialog{
    private _dialogContainer: HTMLElement;
    private _txtName: MDCTextField;
    private _txtEmail: MDCTextField;
    private _txtPhone: MDCTextField;
    private _txtAddress: MDCTextField;
    private _txtNumber: MDCTextField;
    private _txtRef: MDCTextField;
    private _btnSend: HTMLElement;

    constructor() {
        this._dialogContainer = document.querySelector('#cliente-info-dialog') || new HTMLElement();
        this._dialogContainer.innerHTML = "";
        this._dialogContainer.insertAdjacentElement('beforeend', this.getDivContainerInnerHTML("textfield-name", "Nome", "text"));
        this._dialogContainer.insertAdjacentElement('beforeend', this.getDivContainerInnerHTML("textfield-email", "Email", "email"));
        this._dialogContainer.insertAdjacentElement('beforeend', this.getDivContainerInnerHTML("textfield-phone", "Telefone", "text"));
        this._dialogContainer.insertAdjacentElement('beforeend', this.getDivContainerInnerHTML("textfield-address", "Endereço", "text"));
        this._dialogContainer.insertAdjacentElement('beforeend', this.getDivContainerInnerHTML("textfield-number", "Número da Casa", "text"));
        this._dialogContainer.insertAdjacentElement('beforeend', this.getDivContainerInnerHTML("textfield-ref", "Referência", "text"));

        this._txtName = new MDCTextField(document.getElementById('textfield-name') || new HTMLElement());
        this._txtEmail = new MDCTextField(document.getElementById('textfield-email') || new HTMLElement());
        this._txtPhone = new MDCTextField(document.getElementById('textfield-phone') || new HTMLElement());
        this._txtAddress = new MDCTextField(document.getElementById('textfield-address') || new HTMLElement());
        this._txtNumber = new MDCTextField(document.getElementById('textfield-number') || new HTMLElement());
        this._txtRef = new MDCTextField(document.getElementById('textfield-ref') || new HTMLElement());

        this._btnSend = document.getElementById('button-dialog-submit') || new HTMLElement();

        new MDCRipple(this._btnSend);
        this._txtPhone.root.addEventListener('input',  (event) => {
            let input = event.target as HTMLInputElement;
            input.value = this.formatPhoneNumber(input.value);
        });
    }

    private formatPhoneNumber(value: string): string {
        if (!value) return value;
        const phoneNumber = value.replace(/[^\d]/g, '');
        const phoneNumberLength = phoneNumber.length;
        if (phoneNumberLength < 4) return phoneNumber;
        if (phoneNumberLength < 7) {
            return `(${phoneNumber.slice(0, 2)}) ${phoneNumber.slice(2)}`;
        }
        return `(${phoneNumber.slice(0, 2)}) ${phoneNumber.slice(2, 7)}-${phoneNumber.slice(7, 11)}`;
    }

    private getDivContainerInnerHTML(id: string, label: string, type: string): HTMLElement {
        const el = document.createElement('label');
        el.className = "mdc-text-field mdc-text-field--outlined";
        el.id = id;
        el.innerHTML = `
        <span class="mdc-notched-outline">
            <span class="mdc-notched-outline__leading"></span>
            <span class="mdc-notched-outline__notch">
              <span class="mdc-floating-label" id="${id}-label">${label}</span>
            </span>
            <span class="mdc-notched-outline__trailing"></span>
          </span>
          <input type="${type}" class="mdc-text-field__input" aria-labelledby="${id}-label">
        `;
        return el
    }

    public getCustomerData(): CustomerData {
        return {
            nome: this._txtName.getDefaultFoundation().getValue(),
            endereco: this._txtAddress.getDefaultFoundation().getValue(),
            numeroTelefone: this._txtPhone.getDefaultFoundation().getValue(),
            numeroCasa: this._txtNumber.getDefaultFoundation().getValue(),
            referenciaCasa: this._txtRef.getDefaultFoundation().getValue(),
            email: this._txtEmail.getDefaultFoundation().getValue()
        }
    }

    public checkCustomerData(): boolean {
        let isValid = true;

        if (!this._txtName.getDefaultFoundation().getValue().trim() || this._txtName.getDefaultFoundation().getValue().trim() === "" || this._txtName.getDefaultFoundation().getValue().trim() === undefined) {
            isValid = false;
            this._txtName.getDefaultFoundation().setValid(false);
        }
        if (!this._txtAddress.getDefaultFoundation().getValue().trim() || this._txtAddress.getDefaultFoundation().getValue().trim() === "" || this._txtAddress.getDefaultFoundation().getValue().trim() === undefined) {
            isValid = false;
            this._txtAddress.getDefaultFoundation().setValid(false);
        }
        if (!this._txtPhone.getDefaultFoundation().getValue().trim() || this._txtPhone.getDefaultFoundation().getValue().trim() === "" || this._txtPhone.getDefaultFoundation().getValue().trim() === undefined) {
            isValid = false;
            this._txtPhone.getDefaultFoundation().setValid(false);
        }
        if (!this._txtNumber.getDefaultFoundation().getValue().trim() || this._txtNumber.getDefaultFoundation().getValue().trim() === "" || this._txtNumber.getDefaultFoundation().getValue().trim() === undefined) {
            isValid = false;
            this._txtNumber.getDefaultFoundation().setValid(false);
        }
        if (!this._txtRef.getDefaultFoundation().getValue().trim() || this._txtRef.getDefaultFoundation().getValue().trim() === "" || this._txtRef.getDefaultFoundation().getValue().trim() === undefined) {
            isValid = false;
            this._txtRef.getDefaultFoundation().setValid(false);
        }
        if (!this._txtEmail.getDefaultFoundation().getValue().trim() || this._txtEmail.getDefaultFoundation().getValue().trim() === "" || this._txtEmail.getDefaultFoundation().getValue().trim() === undefined) {
            isValid = false;
            this._txtEmail.getDefaultFoundation().setValid(false);
        }

        return isValid
    }

    get txtName(): MDCTextField {
        return this._txtName;
    }

    get txtEmail(): MDCTextField {
        return this._txtEmail;
    }

    get txtPhone(): MDCTextField {
        return this._txtPhone;
    }

    get txtAddress(): MDCTextField {
        return this._txtAddress;
    }

    get txtNumber(): MDCTextField {
        return this._txtNumber;
    }

    get txtRef(): MDCTextField {
        return this._txtRef;
    }

    get btnSend(): HTMLElement {
        return this._btnSend;
    }
}

export class Customer {
    private _customerData: CustomerData;
    private _customerDataLocal: CustomerDataLocal | null;

    constructor(customerData: CustomerData) {
        this._customerData = customerData;
        this._customerDataLocal = null;
    }

    public syncLocal() {
        let customers: CustomerDataLocal[] | null = this.getLocalCustomers();

        if (customers && this._customerDataLocal) {
            customers.push(this._customerDataLocal);
        } else if (this._customerDataLocal) {
            customers = [this._customerDataLocal];
        } else {
            console.error("Error while saving customers");
        }

        if (this._customerDataLocal) {
            localStorage.setItem('customer-data', JSON.stringify(customers));
        }
    }

    private getLocalCustomers(): CustomerDataLocal[] | null {
        let customers: CustomerDataLocal[] | null;
        try {
            customers = JSON.parse(localStorage.getItem('customer-data') || "");
        } catch (e){
            customers = null;
        }

        return customers
    }

    public checkCustomer(): CustomerDataLocal | null {
        const customers: CustomerDataLocal[] | null = this.getLocalCustomers();
        let customer: CustomerDataLocal = {
            idCliente: '',
            nome: '',
            endereco: '',
            numeroTelefone: '',
            numeroCasa: '',
            referenciaCasa: '',
            email: ''
        };

        if (customers) {
            let found: boolean = false;
            customers.forEach((item) => {
                if (item.email === this._customerData.email) {
                    customer = item;
                    found = true;
                }
            });

            if (found) {
                this._customerDataLocal = customer;
                return customer
            } else {
                return null
            }
        } else {
            return null
        }
    }

    public async addNewCustomer(): Promise<boolean> {
        const res = await fetch(`${ApiHost}/cliente/new`, {
            method: "POST",
            body: JSON.stringify(this._customerData),
            headers: {
                "Content-Type": "application/json"
            }
        });

        const resJson = await res.json();

        if (!resJson["error"]){
            this._customerDataLocal = {
                idCliente: resJson["message"],
                nome: this._customerData.nome,
                endereco: this._customerData.endereco,
                numeroTelefone: this.stripPhoneNumberFormatting(this._customerData.numeroTelefone),
                numeroCasa: this._customerData.numeroCasa,
                referenciaCasa: this._customerData.referenciaCasa,
                email: this._customerData.email
            }

            this.syncLocal();

            return true
        } else {
            return false
        }
    }

    public async updateCustomer(customer: CustomerDataLocal): Promise<boolean> {
        customer.nome = this._customerData.nome;
        customer.endereco = this._customerData.endereco;
        customer.numeroTelefone = this._customerData.numeroTelefone;
        customer.numeroCasa = this._customerData.numeroCasa;
        customer.referenciaCasa = this._customerData.referenciaCasa;
        customer.email = this._customerData.email;

        const res = await fetch(`${ApiHost}/cliente/${customer.idCliente}`, {
            method: "PUT",
            body: JSON.stringify(customer),
            headers: {
                "Content-Type": "application/json"
            }
        });

        const resJson = await res.json();

        if (!resJson["error"]) {
            this._customerDataLocal = customer;
            this.syncLocal();

            return true
        } else {
            return false
        }
    }

    get customerDataLocal(): CustomerDataLocal | null {
        return this._customerDataLocal;
    }

    private stripPhoneNumberFormatting(value: string): string {
        return value.replace(/[^\d]/g, '');
    }
}