import {ApiHost} from "./vars";
import {MDCSnackbar} from '@material/snackbar';

const snackbar = new MDCSnackbar(document.querySelector('.mdc-snackbar'));
const fullscreenDiv = document.getElementById('fullscreen-div');
const modal = document.getElementById('modal');
const addBtn = document.getElementById('add-btn');
const listBtn = document.getElementById('list-btn');
const closeBtn = document.getElementById('close-btn');
const modalBtns = document.getElementById('modal-btns');
const form = document.getElementById('modal');
const productList = document.querySelector('#product-list tbody');

let id = productName = status = price = costPrice = quantity = "";

// Produto
class Product {
    constructor(id, name, status, price, costPrice, quantity) {
        this.id = id;
        this.name = name;
        this.status = status;
        this.price = price;
        this.costPrice = costPrice;
        this.quantity = quantity;
    }

    static getProducts() {
        let products;
        if (localStorage.getItem('products') === null) {
            products = [];
        } else {
            products = JSON.parse(localStorage.getItem('products'));
        }
        return products;
    }

    static addProduct(product) {
        const products = Product.getProducts();
        products.push(product);
        localStorage.setItem('products', JSON.stringify(products));
    }

    static deleteProduct(id) {
        const products = Product.getProducts();
        products.forEach((product, index) => {
            if (product.id == id) {
                products.splice(index, 1);
            }
        });
        localStorage.setItem('products', JSON.stringify(products));
        form.reset();
        UI.closeModal();
        productList.innerHTML = "";
        UI.showProductList();
    }

    static updateProduct(item) {
        const products = Product.getProducts();
        products.forEach(product => {
            if (product.id == item.id) {
                product.name = item.name;
                product.status = item.status;
                product.price = item.price;
                product.costPrice = item.costPrice;
                product.quantity = item.quantity;

                localStorage.setItem('products', JSON.stringify(products));
                productList.innerHTML = "";
                UI.showProductList();
            }
        });
    }
}

class UI {
    static showProductList() {
        const products = Product.getProducts();
        products.forEach(product => UI.addToProductList(product));
    }

    static addToProductList(product) {
        const tableRow = document.createElement('tr');
        tableRow.setAttribute('data-id', product.id);
        tableRow.innerHTML = `
            <td>${product.id}</td>
            <td>${product.name}</td>
            <td>${product.status}</td>
            <td>${product.price}</td>
            <td>${product.costPrice}</td>
            <td>${product.quantity}</td>
        `;
        productList.appendChild(tableRow);
    }

    static showModalData(id) {
        const products = Product.getProducts();
        products.forEach(product => {
            if (product.id == id) {
                form.id.value = product.id;
                form.name.value = product.name;
                form.status.value = product.status;
                form.price.value = product.price;
                form.cost_price.value = product.costPrice;
                form.quantity.value = product.quantity;

                document.getElementById('modal-title').innerHTML = "Editar Produto";

                document.getElementById('modal-btns').innerHTML = `
                    <button type="submit" id="update-btn" data-id="${id}">Editar Geladinho</button>
                    <button type="button" id="delete-btn" data-id="${id}">Remover Geladinho</button>
                `;
            }
        });
    }

    static showModal() {
        modal.style.display = "block";
        fullscreenDiv.style.display = "block";
    }

    static closeModal() {
        modal.style.display = "none";
        fullscreenDiv.style.display = "none";
    }
}

window.addEventListener('DOMContentLoaded', () => {
    eventListeners();
    UI.showProductList();
});

function eventListeners() {
    addBtn.addEventListener('click', () => {
        form.reset();
        document.getElementById('modal-title').innerHTML = "Adicionar Produto";
        UI.showModal();
        document.getElementById('modal-btns').innerHTML = `
            <button type="submit" id="save-btn">Salvar</button>
        `;
    });

    closeBtn.addEventListener('click', UI.closeModal);

    listBtn.addEventListener('click', () => {
        location.reload();
    });

    //adicionar
    modalBtns.addEventListener('click', (event) => {
        event.preventDefault();
        if (event.target.id == "save-btn") {
            let isFormValid = getFormData();
            if (!isFormValid) {
                form.querySelectorAll('input').forEach(input => {
                    setTimeout(() => {
                        input.classList.remove('errorMsg');
                    }, 1500);
                });
            } else {
                let allProducts = Product.getProducts();
                let lastProductId = allProducts.length > 0 ? allProducts[allProducts.length - 1].id : 0;
                lastProductId++;

                const productItem = new Product(lastProductId, form.name.value, form.status.value, form.price.value, form.cost_price.value, form.quantity.value);
                Product.addProduct(productItem);
                UI.closeModal();
                UI.addToProductList(productItem);
                form.reset();
            }
        }
    });

    // table row
    productList.addEventListener('click', (event) => {
        UI.showModal();
        let trElement;
        if (event.target.parentElement.tagName == "TD") {
            trElement = event.target.parentElement.parentElement;
        }

        if (event.target.parentElement.tagName == "TR") {
            trElement = event.target.parentElement;
        }

        let viewID = trElement.dataset.id;
        UI.showModalData(viewID);
    });

    // deletar produto
    modalBtns.addEventListener('click', (event) => {
        if (event.target.id == 'delete-btn') {
            Product.deleteProduct(event.target.dataset.id);
        }
    });

    // atualizar produto
    modalBtns.addEventListener('click', (event) => {
        event.preventDefault();
        if (event.target.id == "update-btn") {
            let id = event.target.dataset.id;
            let isFormValid = getFormData();
            if (!isFormValid) {
                form.querySelectorAll('input').forEach(input => {
                    setTimeout(() => {
                        input.classList.remove('errorMsg');
                    }, 1500);
                });
            } else {
                const productItem = new Product(id, form.name.value, form.status.value, form.price.value, form.cost_price.value, form.quantity.value);
                Product.updateProduct(productItem);
                UI.closeModal();
                form.reset();
            }
        }
    });
}

// form data
function getFormData() {
    let inputValidStatus = [];

    id = form.id.value;
    name = form.name.value;
    status = form.status.value;
    price = form.price.value;
    costPrice = form.cost_price.value;
    quantity = form.quantity.value;

    // validacao

    return inputValidStatus.includes(false) ? false : true;
}
document.addEventListener('DOMContentLoaded', () => {
    const modal = document.getElementById('modal');
    const form = document.querySelector('form');
    const addBtn = document.getElementById('add-btn');
    const closeBtn = document.getElementById('close-btn');
    const productList = document.getElementById('product-list').querySelector('tbody');
    const modalTitle = document.getElementById('modal-title');
    let produtoParaAtualizar = null;

    addBtn.addEventListener('click', () => {
        modalTitle.textContent = 'Adicionar Geladinho';
        produtoParaAtualizar = null;
        form.reset();
        modal.style.display = 'block';
    });

    closeBtn.addEventListener('click', () => {
        form.reset();
        modal.style.display = 'none';
    });

    form.addEventListener('submit', (e) => {
        e.preventDefault();
        const formData = new FormData(form);
        const produto = {
            idGeladinho: produtoParaAtualizar ? produtoParaAtualizar : null,
            nomeGeladinho: formData.get('name'),
            descricaoGeladinho: formData.get('description'),
            statusGeladinho: formData.get('status') === 'ativo',
            precoGeladinho: parseFloat(formData.get('price')),
            imagemGeladinho: formData.get('image'), // Ajuste conforme necessário para upload de arquivo
            quantidadeEstoque: parseInt(formData.get('quantity'), 10)
        };

        if (produtoParaAtualizar) {
            atualizarProduto(produto);
        } else {
            enviarProduto(produto);
        }
    });

    function enviarProduto(produto) {
        fetch(`${ApiHost}/api/geladinhos`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(produto)
        }).then(response => response.ok && listarProdutos())
            .catch(error => console.error('Falha ao enviar geladinho:', error))
            .finally(() => {
                form.reset();
                modal.style.display = 'none';
            });
    }

    function atualizarProduto(produto) {
        fetch(`${ApiHost}/api/geladinhos/${produto.idGeladinho}`, {
            method: 'PUT',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(produto)
        }).then(response => response.ok && listarProdutos())
            .catch(error => console.error('Falha ao atualizar geladinho:', error))
            .finally(() => {
                form.reset();
                modal.style.display = 'none';
                produtoParaAtualizar = null;
            });
    }

    function listarProdutos() {
        fetch(`${ApiHost}/api/geladinhos`)
            .then(response => response.json())
            .then(produtos => {
                productList.innerHTML = '';
                produtos.forEach(produto => {
                    const row = document.createElement('tr');
                    Object.entries(produto).forEach(([key, value]) => {
                        const cell = document.createElement('td');
                        if(key === 'imagemGeladinho' && value) {
                            const img = document.createElement('img');
                            img.src = value; // Ajuste o caminho da imagem conforme necessário
                            img.alt = 'Imagem do Geladinho';
                            img.style.width = '50px';
                            cell.appendChild(img);
                        } else {
                            cell.textContent = value;
                        }
                        row.appendChild(cell);
                    });
                    const actionCell = document.createElement('td');
                    const updateBtn = document.createElement('button');
                    updateBtn.textContent = 'Atualizar';
                    updateBtn.addEventListener('click', () => abrirModalParaAtualizar(produto.idGeladinho));
                    actionCell.appendChild(updateBtn);

                    const deleteBtn = document.createElement('button');
                    deleteBtn.textContent = 'Deletar';
                    deleteBtn.addEventListener('click', () => deletarProduto(produto.idGeladinho));
                    actionCell.appendChild(deleteBtn);

                    row.appendChild(actionCell);
                    productList.appendChild(row);
                });
            })
            .catch(error => console.error('Erro ao listar geladinhos:', error));
    }

    function abrirModalParaAtualizar(idGeladinho) {
        fetch(`${ApiHost}/api/geladinhos/${idGeladinho}`)
            .then(response => response.json())
            .then(produto => {
                modalTitle.textContent = 'Atualizar Geladinho';
                produtoParaAtualizar = idGeladinho;
                // Preenche os campos do formulário com os dados do produto
                form.name.value = produto.nomeGeladinho;
                form.description.value = produto.descricaoGeladinho;
                form.status.value = produto.statusGeladinho ? 'ativo' : 'inativo';
                form.price.value = produto.precoGeladinho;
                form.quantity.value = produto.quantidadeEstoque;
                // Ajuste para campo de imagem conforme necessário
                modal.style.display = 'block';
            });
    }

    function deletarProduto(idGeladinho) {
        fetch(`${ApiHost}/api/geladinhos/${idGeladinho}`, {
            method: 'DELETE'
        }).then(response => response.ok && listarProdutos())
            .catch(error => console.error('Falha ao deletar geladinho:', error));
    }

    listarProdutos();
});
