import { ApiHost } from "@/js/vars";
import moment from "moment";
import {MDCSnackbar} from '@material/snackbar';

const snackbar = new MDCSnackbar(document.querySelector('.mdc-snackbar'));

document.addEventListener("DOMContentLoaded", async function() {
    await carregarAvaliacoes();
});

async function carregarAvaliacoes() {
    const resposta = await fetch(`${ApiHost}/feedback`);
    const dados = await resposta.json();
    const avaliacoes = dados["feedbackList"];

    const tabelaCorpo = document.querySelector('.order-table tbody');
    tabelaCorpo.innerHTML = '';

    avaliacoes.forEach(avaliacao => {
        const linha = document.createElement('tr');
        linha.innerHTML = `
            <td>${avaliacao.name}</td>
            <td>${moment.unix(avaliacao.date).format('DD-MM-YYYY HH:mm:ss')}</td>
            <td>${avaliacao.rating}</td>
            <td>${avaliacao.texto}</td>
            <td><button class="delete-btn" data-clientid="${avaliacao.clientId}"><i class="fa fa-trash"></i> Excluir</button></td>
        `;
        tabelaCorpo.appendChild(linha);
    });

    adicionarEventosDeExclusao();
}

function adicionarEventosDeExclusao() {
    const botoesExcluir = document.querySelectorAll('.delete-btn');
    botoesExcluir.forEach(botao => {
        botao.addEventListener('click', function() {
            const clientId = this.getAttribute('data-clientid');
            excluirAvaliacao(clientId);
        });
    });
}

async function excluirAvaliacao(clientId) {
    const response = await fetch(`${ApiHost}/feedback/${clientId}`, {
        method: 'DELETE'
    });

    if (response.ok) {
        snackbar.open();
        snackbar.labelText = "Avaliação excluída com sucesso!";
        await carregarAvaliacoes();
    } else {
        const responseData = await response.json();
        snackbar.open();
        snackbar.labelText = "Erro ao excluir avaliação";
    }
}

