import {ApiHost} from "./vars";
import {MDCSnackbar} from '@material/snackbar';

const snackbar = new MDCSnackbar(document.querySelector('.mdc-snackbar'));

document.addEventListener('DOMContentLoaded', () => {
    const modal = document.getElementById('modal');
    const form = document.querySelector('form');
    const addBtn = document.getElementById('add-btn');
    const closeBtn = document.getElementById('close-btn');
    const productList = document.getElementById('product-list').querySelector('tbody');
    const modalTitle = document.getElementById('modal-title');
    let produtoParaAtualizar = null;

    addBtn.addEventListener('click', () => {
        modalTitle.textContent = 'Adicionar Geladinho';
        produtoParaAtualizar = null;
        form.reset();
        modal.style.display = 'block';
    });

    closeBtn.addEventListener('click', () => {
        form.reset();
        modal.style.display = 'none';
    });

    form.addEventListener('submit', (e) => {
        e.preventDefault();
        const formData = new FormData(form);

        let inputImage = document.getElementById("inputImage");

        const produto = {
            nomeGeladinho: formData.get('name'),
            descricaoGeladinho: formData.get('description'),
            statusGeladinho: formData.get('status') === 'ativo',
            precoGeladinho: parseFloat(formData.get('price')),
            imagemGeladinho: inputImage.files[0],
            quantidadeEstoque: parseInt(formData.get('quantity'), 10)
        };

        if (produtoParaAtualizar) {
            atualizarProduto(produtoParaAtualizar, produto);
        } else {
            enviarProduto(produto);
        }
    });

    async function enviarProduto(produto) {
        produto.idGeladinho = crypto.randomUUID();
        fetch(`${ApiHost}/produto`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(produto)
        }).then(response => response.ok && listarProdutos())
            .catch(error => console.error('Falha ao enviar produto:', error))
            .finally(() => {
                form.reset();
                modal.style.display = 'none';
            });

        let inputImage = document.getElementById("inputImage");

        const formData = new FormData();
        formData.append("file", inputImage.files[0]);

        await fetch(`${ApiHost}/produto/image/${produto.idGeladinho}`, {
            method: "POST",
            body: formData
        }).then(response => response.ok && listarProdutos())
            .catch(error => console.error('Falha ao enviar imagem do produto:', error))
            .finally(() => {
                console.log("Image sent.")
            });
    }

    async function atualizarProduto(id, produto) {
        const formData = new FormData();

        formData.append('nomeGeladinho', produto.nomeGeladinho);
        formData.append('descricaoGeladinho', produto.descricaoGeladinho);
        formData.append('statusGeladinho', produto.statusGeladinho ? 'true' : 'false'); // Assumindo que o status é um booleano
        formData.append('precoGeladinho', produto.precoGeladinho.toString());
        formData.append('quantidadeEstoque', produto.quantidadeEstoque.toString());
        /*
        if (produto.imagemGeladinho && produto.imagemGeladinho.size > 0) {
            formData.append('imagemGeladinho', produto.imagemGeladinho);
        }

         */

        const produtoObj = {
            nomeGeladinho: produto.nomeGeladinho,
            descricaoGeladinho: produto.descricaoGeladinho,
            statusGeladinho: produto.statusGeladinho,
            precoGeladinho: produto.precoGeladinho,
            quantidadeEstoque: produto.quantidadeEstoque
        }

        let inputImage = document.getElementById("inputImage");

        if (inputImage.files[0]) {
            const imageData = new FormData();
            imageData.append("file", inputImage.files[0]);

            await fetch(`${ApiHost}/produto/image/${id}`, {
                method: "POST",
                body: imageData
            }).then(response => response.ok && listarProdutos())
                .catch(error => console.error('Falha ao enviar imagem do produto:', error))
                .finally(() => {
                    console.log("Image sent.")
                });
        }

        await fetch(`${ApiHost}/produto/${id}`, {
            method: 'PUT',
            body: JSON.stringify(produtoObj),
            headers: {
                "Content-Type": "application/json"
            }
        }).then(response => {
            if (response.ok) {
                listarProdutos();
                modal.style.display = 'none';
                snackbar.open();
                snackbar.labelText = "Produto atualizado com sucesso!";
            } else {
                console.error('Falha ao atualizar produto');
                snackbar.open();
                snackbar.labelText = "Erro ao atualizar produto";
            }
        }).catch(error => {
            console.error('Falha ao atualizar produto:', error);
            snackbar.open();
            snackbar.labelText = "Erro ao atualizar produto";
        }).finally(() => {
            form.reset();
            produtoParaAtualizar = null;
        });
    }


    function listarProdutos() {
        fetch(`${ApiHost}/produtos`)
            .then(response => response.json())
            .then(produtos => {
                productList.innerHTML = '';
                produtos["produtos"].forEach(produto => {
                    const row = document.createElement('tr');
                    let count = 0;
                    Object.keys(produto).forEach(key => {
                        if (count === 1) {
                            const cell = document.createElement("img");
                            cell.src = `${ApiHost}/produto/image/${produto.idGeladinho}`;
                            cell.className = "imagem-produto-lista";
                            row.appendChild(cell);
                        }
                        const cell = document.createElement('td');
                        cell.textContent = produto[key];
                        row.appendChild(cell);
                        count++;
                    });
                    addActionButton(row, produto.idGeladinho, 'Atualizar', abrirModalParaAtualizar);
                    addActionButton(row, produto.idGeladinho, 'Deletar', deletarProduto);
                    productList.appendChild(row);
                });
            })
            .catch(error => console.error('Erro ao listar produtos:', error));
    }

    function abrirModalParaAtualizar(id) {
        fetch(`${ApiHost}/produto/${id}`)
            .then(response => response.json())
            .then(produto => {
                const produtoObj = produto["produtos"][0];
                modalTitle.textContent = 'Atualizar Geladinho';
                form.name.value = produtoObj.nomeGeladinho;
                form.description.value = produtoObj.descricaoGeladinho;
                //form.status.value = produtoObj.statusGeladinho ? 'ativo' : 'inativo';
                form.price.value = produtoObj.precoGeladinho;
                form.quantity.value = produtoObj.quantidadeEstoque;
                produtoParaAtualizar = id;
                modal.style.display = 'block';
            });
    }

    function deletarProduto(id) {
        fetch(`${ApiHost}/produto/${id}`, {
            method: 'DELETE'
        }).then(response => response.ok && listarProdutos())
            .catch(error => console.error('Falha ao deletar produto:', error));
    }

    function addActionButton(row, id, text, action) {
        const actionCell = document.createElement('td');
        const actionBtn = document.createElement('button');
        actionBtn.textContent = text;
        actionBtn.addEventListener('click', () => action(id));
        actionCell.appendChild(actionBtn);
        row.appendChild(actionCell);
    }

    listarProdutos();
});
