import {ApiHost} from "@/js/vars";

document.addEventListener('DOMContentLoaded', () => {
    const taxaForm = document.getElementById('taxaForm');
    const deliveryFeeInput = document.getElementById('deliveryFee');
    const taxaTableBody = document.querySelector('#taxaTable tbody');
    const taxaIdInput = document.getElementById('taxaId');

    taxaForm.addEventListener('submit', (e) => {
        e.preventDefault();
        const deliveryFee = parseFloat(deliveryFeeInput.value);
        const id = taxaIdInput.value;

        if (id) {
            updateTaxa(id, { deliveryFee });
        } else {
            createTaxa({ deliveryFee });
        }
    });

    function fetchTaxas() {
        fetch(`${ApiHost}/deliveryFee/listar`)
            .then(response => response.json())
            .then(data => {
                taxaTableBody.innerHTML = '';
                data.deliveryFees.forEach(taxa => {
                    const row = document.createElement('tr');
                    row.innerHTML = `
                        <td>${taxa.id}</td>
                        <td>${taxa.deliveryFee.toFixed(2)}</td>
                        <td>
                            <button onclick="editTaxa(${taxa.id}, ${taxa.deliveryFee})">Editar</button>
                            <button onclick="deleteTaxa(${taxa.id})">Deletar</button>
                        </td>
                    `;
                    taxaTableBody.appendChild(row);
                });
            })
            .catch(error => console.error('Error:', error));
    }

    function createTaxa(taxa) {
        fetch(`${ApiHost}/deliveryFee/registrar`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify(taxa)
        })
            .then(response => response.json())
            .then(data => {
                alert(data.message);
                fetchTaxas();
                taxaForm.reset();
            })
            .catch(error => console.error('Error:', error));
    }

    function updateTaxa(id, taxa) {
        fetch(`${ApiHost}/deliveryFee/editar/${id}`, {
            method: 'PUT',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify(taxa)
        })
            .then(response => response.json())
            .then(data => {
                alert(data.message);
                fetchTaxas();
                taxaForm.reset();
                taxaIdInput.value = '';
            })
            .catch(error => console.error('Error:', error));
    }

    window.deleteTaxa = function(id) {
        fetch(`${ApiHost}/deliveryFee/deletar/${id}`, {
            method: 'DELETE'
        })
            .then(response => response.json())
            .then(data => {
                alert(data.message);
                fetchTaxas();
            })
            .catch(error => console.error('Error:', error));
    }

    window.editTaxa = function(id, deliveryFee) {
        taxaIdInput.value = id;
        deliveryFeeInput.value = deliveryFee;
    }

    fetchTaxas();
});
