import {ApiHost} from "@/js/vars";
import {CustomerModel, FeedbackModel, Settings, SettingsModel} from "@/js/declarations";
import Swiper from "swiper";

async function loadFeedbacks() {
    const ctFeedbacks = document.getElementById("ctFeedbacks");
    const feedbacks = await fetch(`${ApiHost}/feedback/public`).then((res) => res.json()) as FeedbackModel;
    if (!feedbacks.error && feedbacks.feedbackList.length > 0 && ctFeedbacks) {
        ctFeedbacks.innerHTML = "";
        for (const feedback of feedbacks.feedbackList) {
            const customerName = await fetch(`${ApiHost}/cliente/public/${feedback.clientId}`).then((res) => res.json()) as CustomerModel;
            let stars = "";
            for (let i = 0; i < feedback.rating; i++) {
                stars += `<i class="fas fa-star"></i>`;
            }
            ctFeedbacks.innerHTML += `
            <div class="swiper-slide slide">
                <h3>${customerName.message}</h3>
                <p>${feedback.texto}</p>
                <div class="stars">
                    ${stars}
                </div>
            </div>
            `;
        }

        let swiper = new Swiper(".review-slider", {
            loop:true,
            spaceBetween: 30,
            centeredSlides: true,
            autoplay: {
                delay: 5500,
                disableOnInteraction: false,
            },
            pagination: {
                el: ".swiper-pagination",
            },
        });
    } else {
        const review = document.getElementById("review");
        review ? review.remove() : null;
    }
}

function loadData() {
    const pbInnerContainer = document.querySelector(".pb-inner-ct") as HTMLElement | null;
    const pbContainer = document.querySelector("#progress-bar-container") as HTMLElement | null;
    const mainContent = document.querySelector("#main-content") as HTMLElement | null;
    let validEmail = true;
    let validPhone = true;
    let validIg = true;
    let validWa = true;
    fetch(`${ApiHost}/settings/site/all`).then((res) => res.json()).then((res: SettingsModel[]) => {
        if (checkValidSetting(res)) {
            loadDetails();
            res.forEach((setting) => {
                switch (setting.setting.key) {
                    case "site_header": {
                        const siteHeader = document.getElementById("site-header"); siteHeader? siteHeader.innerHTML = setting.setting.value : null;
                        break;
                    }
                    case "site_name": {
                        document.title = setting.setting.value;
                        const siteName = document.getElementById("site-name"); siteName? siteName.innerHTML = setting.setting.value : null;
                        const siteNameFooter = document.getElementById("site-name-footer"); siteNameFooter? siteNameFooter.innerHTML = setting.setting.value : null;
                        break;
                    }
                    case "site_description": {
                        const siteDescription = document.getElementById("site-description"); siteDescription? siteDescription.innerHTML = setting.setting.value.replace(/\n/g, '<br>') : null;
                        break;
                    }
                    case "support_email": {
                        const supportEmail = document.getElementById("support-email");
                        if (supportEmail) {
                            if (!setting.setting.value || setting.setting.value === "" || setting.setting.value === null || setting.setting.value === undefined) {
                                supportEmail.remove();
                                validEmail = false;
                            } else {
                                supportEmail.innerHTML = `<i class="fas fa-envelope"></i> ${setting.setting.value}`;
                                supportEmail.setAttribute("href", `mailto:${setting.setting.value}`);
                                supportEmail.setAttribute("target", "_blank");
                            }
                        }
                        break;
                    }
                    case "support_phone": {
                        const supportPhone = document.getElementById("support-phone");
                        if (supportPhone) {
                            if (!setting.setting.value || setting.setting.value === "" || setting.setting.value === null || setting.setting.value === undefined) {
                                supportPhone.remove();
                                validPhone = false;
                            } else {
                                supportPhone.innerHTML = ` <i class="fas fa-phone"></i> +55 ${formatPhoneNumber(setting.setting.value)}`;
                                supportPhone.setAttribute("href", `tel:55${setting.setting.value}`);
                                supportPhone.setAttribute("target", "_blank");
                            }
                        }
                        break;
                    }
                    case "social_instagram": {
                        const socialInstagram = document.getElementById("social-instagram");
                        if (socialInstagram) {
                            if (!setting.setting.value || setting.setting.value === "" || setting.setting.value === null || setting.setting.value === undefined) {
                                socialInstagram.remove();
                                validIg = false;
                            } else {
                                socialInstagram.setAttribute("href", `https://instagram.com/${setting.setting.value}/`);
                                socialInstagram.setAttribute("target", "_blank");
                            }
                        }
                        break;
                    }
                    case "social_whatsapp": {
                        const socialWhatsApp = document.getElementById("social-whatsapp");
                        if (socialWhatsApp) {
                            if (!setting.setting.value || setting.setting.value === "" || setting.setting.value === null || setting.setting.value === undefined) {
                                socialWhatsApp.remove();
                                validWa = false;
                            } else {
                                socialWhatsApp.setAttribute("href", `https://wa.me/55${setting.setting.value}`);
                                socialWhatsApp.setAttribute("target", "_blank");
                            }
                        }
                        break;
                    }
                }
            });

            if (!validIg && !validWa) {
                const socialContainer = document.getElementById("container-social");
                socialContainer? socialContainer.remove() : null;
            }
            if (!validEmail && !validPhone) {
                const supportContainer = document.getElementById("container-support");
                supportContainer? supportContainer.remove() : null;
            }

            if (pbContainer && pbInnerContainer && mainContent) {
                pbContainer.classList.add("hidden");
                setTimeout(() => {
                    pbContainer.style.display = "none";
                    mainContent.style.display = "block";
                }, 500);
            }
        } else {
            if (pbContainer && pbInnerContainer && mainContent) {
                pbInnerContainer.innerHTML = `
                <span class="material-icons-outlined" style="height: 60px;width: 60px;font-size: 60px;">handyman</span>
                <h4 class="mdc-typography--headline4">Em Manutenção</h4>
                <p class="mdc-typography--subtitle1">Ainda estamos configurando nosso site, por favor, volte mais tarde.</p>
                `
            }
        }
    });
    const siteImage = document.getElementById("site-image"); siteImage ? siteImage.setAttribute("src", `${ApiHost}/settings/site/image`) : null;
}

function loadDetails() {
    fetch(`${ApiHost}/settings/info`).then((res) => res.json()).then((res: SettingsModel[]) => {
        res.forEach((setting) => {
            switch (setting.setting.key) {
                case "support_email": {
                    const supportEmail = document.getElementById("support-email");
                    if (supportEmail) {
                        if (!setting.setting.value || setting.setting.value === "" || setting.setting.value === null || setting.setting.value === undefined) {
                            supportEmail.remove();
                        } else {
                            supportEmail.innerHTML = `<i class="fas fa-envelope"></i> ${setting.setting.value}`;
                            supportEmail.setAttribute("href", `mailto:${setting.setting.value}`);
                            supportEmail.setAttribute("target", "_blank");
                        }
                    }
                    break;
                }
                case "support_phone": {
                    const supportPhone = document.getElementById("support-phone");
                    if (supportPhone) {
                        if (!setting.setting.value || setting.setting.value === "" || setting.setting.value === null || setting.setting.value === undefined) {
                            supportPhone.remove();
                        } else {
                            supportPhone.innerHTML = ` <i class="fas fa-phone"></i> +55 ${formatPhoneNumber(setting.setting.value)}`;
                            supportPhone.setAttribute("href", `tel:55${setting.setting.value}`);
                            supportPhone.setAttribute("target", "_blank");
                        }
                    }
                    break;
                }
            }
        })
    });
    fetch(`${ApiHost}/settings/social/instagram`).then((res) => res.json()).then((res: SettingsModel) => {
        const socialInstagram = document.getElementById("social-instagram");
        if (socialInstagram) {
            socialInstagram.setAttribute("href", `https://instagram.com/${res.setting.value}/`);
            socialInstagram.setAttribute("target", "_blank");
        }
    });
    fetch(`${ApiHost}/settings/social/whatsapp`).then((res) => res.json()).then((res: SettingsModel) => {
        const socialWhatsApp = document.getElementById("social-whatsapp");
        if (socialWhatsApp) {
            socialWhatsApp.setAttribute("href", `https://wa.me/55${res.setting.value}`);
            socialWhatsApp.setAttribute("target", "_blank");
        }
    });
}

function formatPhoneNumber(value: string): string {
    if (!value) return value;
    const phoneNumber = value.replace(/[^\d]/g, '');
    const phoneNumberLength = phoneNumber.length;
    if (phoneNumberLength < 4) return phoneNumber;
    if (phoneNumberLength < 7) {
        return `(${phoneNumber.slice(0, 2)}) ${phoneNumber.slice(2)}`;
    }
    return `(${phoneNumber.slice(0, 2)}) ${phoneNumber.slice(2, 7)}-${phoneNumber.slice(7, 11)}`;
}

function checkValidSetting(settings: SettingsModel[]): boolean {
    let valid = true;
    settings.forEach((setting) => {
        if (setting.setting.key === "setup_check") {
            if (setting.error) {
                valid = false;
            }
        }
    });
    return valid;
}

loadFeedbacks();
loadData();