import { defineConfig } from 'vite'
import * as path from "path";
import {viteStaticCopy} from "vite-plugin-static-copy";
import tsconfigPaths from "vite-tsconfig-paths";

// https://vitejs.dev/config/
export default defineConfig({
    mode: 'development',
    base: '/',
    root: 'src',
    build: {
        outDir: '../dist',
        rollupOptions: {
            input: {
                main: path.resolve(__dirname, 'src/index.html'),
                gerenciador_estoque: path.resolve(__dirname, 'src/gerenciador_estoque.html'),
                gerenciador_produto: path.resolve(__dirname, 'src/gerenciador_produto.html'),
                avaliacao: path.resolve(__dirname, 'src/avaliacao.html'),
                gerenciador_pedido: path.resolve(__dirname, 'src/gerenciador_pedido.html'),
                estoque_ingredientes: path.resolve(__dirname, 'src/estoque_ingredientes.html'),
                gerenciador_feedbacks: path.resolve(__dirname, 'src/gerenciador_feedbacks.html'),
                gerenciador_taxa: path.resolve(__dirname, 'src/gerenciador_taxa.html'),
                admin: path.resolve(__dirname, 'src/admin.html'),
                template_taxa: path.resolve(__dirname, 'src/template_taxa.html'),
                order_confirmation: path.resolve(__dirname, 'src/order_confirmation.html'),
                template_order_confirmation: path.resolve(__dirname, 'src/template_order_confirmation.html'),
                template_produtos: path.resolve(__dirname, 'src/template_produtos.html'),
                template_produtos_dialog: path.resolve(__dirname, 'src/template_produtos_dialog.html'),
                template_ingredientes: path.resolve(__dirname, 'src/template_ingredientes.html'),
                template_ingredientes_dialog: path.resolve(__dirname, 'src/template_ingredientes_dialog.html'),
                template_pedidos: path.resolve(__dirname, 'src/template_pedidos.html'),
                template_pedidos_dialog: path.resolve(__dirname, 'src/template_pedidos_dialog.html'),
                template_estoque: path.resolve(__dirname, 'src/template_estoque.html'),
                template_estoque_dialog: path.resolve(__dirname, 'src/template_estoque_dialog.html'),
                template_admin_feedback: path.resolve(__dirname, 'src/template_admin_feedback.html'),
                template_admin_feedback_dialog: path.resolve(__dirname, 'src/template_admin_feedback_dialog.html'),
                template_feedback_dialog: path.resolve(__dirname, 'src/template_feedback_dialog.html'),
                template_admin_settings: path.resolve(__dirname, 'src/template_admin_settings.html'),
            }
        }
    },
    plugins: [
        tsconfigPaths(),
        viteStaticCopy({
            targets: [
                {
                    src: path.resolve(__dirname, './node_modules/ionicons/dist/ionicons/svg') + '/[!.]*',
                    dest: './svg',
                },
                {
                    src: path.resolve(__dirname, './node_modules/material-icon-theme/icons') + '/[!.]*',
                    dest: './material-icon-theme',
                }
            ],
        }),
    ],
    resolve: {
        //mainFields: ['module', 'main', 'jsnext:main'],
        //extensions: ['.js', '.mjs', '.json', '.vue'],
        alias: {
            '@': path.resolve(__dirname, './src'),
            '%': path.resolve(__dirname, './node_modules/ionic/core/dist/ionic/svg')
        },
    },
})
